# pytel < https://t.me/kastaid >
# Copyright (C) 2023-present kastaid
#
# This file is a part of < https://github.com/kastaid/pytel/ >
# Please read the GNU Affero General Public License in
# < https://github.com/kastaid/pytel/blob/main/LICENSE/ >.

from asyncio import Lock
from typing import Set

_GCAST_LOCKED: Set[int] = set()
_GUCAST_LOCKED: Set[int] = set()
_GBAN_LOCKED: Set[int] = set()
_UNGBAN_LOCKED: Set[int] = set()

_INVITED_LOCKED: Set[int] = set()
_KICKED_LOCKED: Set[int] = set()

_HELP_ACCEPT: Set[int] = set()
_HELP_LOCK = Lock()

_MENTION_LOCKED: Set[int] = set()

SETMODE_ONLINE: Set[int] = set()
SETMODE_OFFLINE: Set[int] = set()

_PYTEL_UPDATE = Lock()

LOCK_TYPES: dict = {
    "all": "Everything.",
    "messages": "Text, contacts, locations and venues.",
    "media": "Audio files, documents, photos, videos, video notes and voice notes.",
    "others": "Stickers, games, gifs, inline.",
    "links": "Web priview.",
    "polls": "Polling.",
    "info": "Change info.",
    "invite": "Invite users.",
    "pin": "Pinned messages.",
}

_CHARACTER_NAMES = {
    "`": "",
    "*": "",
    "_": "",
    "-": "",
    "~": "",
    "/": "",
    "|": "",
    "[": "",
    "]": "",
    "<": "",
    ">": "",
    "'": "",
    "{": "",
    "}": "",
    ")": "",
    "(": "",
    "’": "",
    "‘": "",
    "=": "",
    "#": "",
    "&": "",
    "+": "",
    "^": "",
    "%": "",
    "°": "",
    ";": "",
    ":": "",
    "?": "",
    "!": "",
    "@": "",
    "¡": "",
    "¿": "",
    "‽": "",
    "♪": "",
    "±": "",
    '″': '',
    "‚": "",
    ".": "",
    "№": "",
    "—": "",
    "–": "",
    "·": "",
}

SIZE_UNITS = [
    "B",
    "KB",
    "MB",
    "GB",
    "TB",
    "PB",
    "EB",
]

OUT_AFK = [
    "Is Alive !!",
    "Is Here !!",
    "Is Back !!",
    "Is Awake !!",
    "Is Awakening !!",
    "Is Online !!",
    "Is Active !!",
    "Is Finally Here !!",
    "Well Done !!",
    "No Longer AFK !!",
    "Is Coming !!",
    "No Longer Offline !!",
    "Back Again !!",
]

STATISTIC_PYTEL = r"""
📊 Data Usage
Upload: {}
Download: {}

💾 Disk Space
Total: {}
Used: {}
Free: {}

📈 Memory Usage
CPU: {}
RAM: {}
SWAP: {}
"""

STATISTIC_DB = r"""
DATABASE ( PYTEL-Premium )

• Database Type: {}
• Database Size: {}
• Database Table Contents:
  › Table: {}
  › Contents: {}

Copyright (C) 2023-present @kastaid
"""

WEATHER_ICONS: dict = {
    "01d": "☀️",
    "02d": "⛅️",
    "03d": "☁️",
    "04d": "☁️",
    "09d": "🌧",
    "10d": "🌦",
    "11d": "⛈" ,
    "13d": "🌨❄️" ,
    "50d": "🌫",
    "01n": "🌕",
    "02n": "☁🌔",
    "03n": "☁️",
    "04n": "️️☁☁️",
    "09n": "🌧",
    "10n": "🌧🌑",
    "11n": "⛈",
    "13n": "🌨❄️",
    "50n": "🌫",
}



# pytel < https://t.me/kastaid >
# Copyright (C) 2023-present kastaid
#
# This file is a part of < https://github.com/kastaid/pytel/ >
# Please read the GNU Affero General Public License in
# < https://github.com/kastaid/pytel/blob/main/LICENSE/ >.

import re
from functools import reduce
from typing import Optional, Union

def replace_all(
    text: str,
    repls: dict,
    regex: bool = False,
) -> str:
    if regex:
        return reduce(
            lambda a, kv: re.sub(*kv, a, flags=re.I),
            repls.items(),
            text
        )
    return reduce(
        lambda a, kv: a.replace(*kv),
        repls.items(),
        text
    )


def normalize_youtube_url(
    url: str
) -> str:
    if not url.lower().startswith(("http://", "https://")):
        url = "https://" + url
    host = url.split("//")[-1].split("/")[0].split("?")[0]
    repls = {
        host: host.lower(),
        "m.": "",
        "music.": "",
        "youtube-nocookie": "youtube",
        "shorts/": "watch?v=",
        "embed/": "watch?v=",
    }
    return replace_all(
        url,
        repls
    ).split("&")[0]


def is_youtube_url(
    url: str
) -> bool:
    pattern = r"^(?:https?://)?((www|m|music)\.|)((youtu\.be/.+)|((youtube|youtube-nocookie)\.com/(watch\?v=|shorts/|embed/).+))$"
    return bool(
        re.match(
            pattern,
            url,
            flags=re.I
        )
    )


def converting_binnary(
    data: str,
    convert: bool= None,
):
    if convert:
        binnary = " ".join(
            format(x, "08b")
            for x in bytearray(
                data, "utf-8"
            )
        )
    else:
        a = "".join(data)
        number = a.split()
        binnary = "".join(
            chr(int(binary, 2))
            for binary in number
        )

    return binnary


def crypto_format(
    num_count: Union[int, float],
    precision=1,
) -> Optional[str]:
    suffixes = [
        "",
        "K",
        " Million",
        " Billion",
        " Trillion",
        " Quadrillion",
    ]
    try:
        m = sum(
            [
                abs(num_count / 1000**x)
                >= 1
                for x in range(
                    1, len(suffixes)
                )
            ]
        )
        return f"{num_count/1000**m:,.{precision}f}{suffixes[m]}"
    except Exception:
        if float(num_count):
            return sizeof_number(float(num_count))
        else:
            return sizeof_number(int(num_count))


def sizeof_number(
    number: Union[int, float]
):
    for unit in [
        "",
        "K",
        " Million",
        " Billion",
    ]:
        if abs(number) < 1000.0:
            return f"{number:0.3f}{unit}"
        number /= 1000.0
    return f"{number:0.3f}{unit}"



# pytel < https://t.me/kastaid >
# Copyright (C) 2023-present kastaid
#
# This file is a part of < https://github.com/kastaid/pytel/ >
# Please read the GNU Affero General Public License in
# < https://github.com/kastaid/pytel/blob/main/LICENSE/ >.

__version__ = "1.2.239"
